@@ -1,4 +1,5 @@
 #include <stdio.h>
+#include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
 #include <signal.h>
@@ -13,7 +14,20 @@
 #endif
 
 #include <unistd.h>
-#include "common.h"
+
+#ifdef HAVE_SSL
+#include <openssl/crypto.h>
+#include <openssl/x509.h>
+#include <openssl/pem.h>
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#include <openssl/rand.h>
+#endif
+
+#include "sbuf.h"
+#include "struct.h"
+#include "send.h"
+
 
 #ifndef VERSION
 #define VERSION "v?.?.?"
@@ -33,7 +47,7 @@ extern int initproxy (confetti * jr);
 
 extern char buffer[];
 
-
+#if 0
 int bnclog (confetti * jr, char *logbuff)
 {
   static long tata;
@@ -43,7 +57,7 @@ int bnclog (confetti * jr, char *logbuff)
   if (jr->logf != 1)
     return 0;
   tata = time ((long *) 0);
-  snprintf (tmpa, 40, "%s", ctime (&tata));
+  sprintf(tmpa, "%.26s", ctime (&tata));
   for (p = 0; p < 40; p++)
   {
     switch (tmpa[p])
@@ -61,6 +75,8 @@ int bnclog (confetti * jr, char *logbuff)
   fflush (jr->logfile);
   return 0;
 }
+#endif
+
 struct deathwish
 {
 	int reason;
@@ -79,6 +95,10 @@ struct deathwish bncerr[] =
   { SELECTERR, "Select call returned in error" },
   { DOWNER, "Shut down by Supervisor" },
   { KILLED, "I wuz shot down!" },
+#ifdef HAVE_SSL
+  { PUBLICCERTERR, "There is an issue with the public certificate"},
+  { PRIVATECERTERR, "There is an issue with the private certificate"},
+#endif
   { 0, "Died for an unknown reason" }
 };
 
@@ -141,11 +161,12 @@ int main (int argc, char **argv)
 		strncat (conffile, ".conf", PACKETBUFF);
 		conffile[PACKETBUFF]='\0';
 	}
-	printf ("Irc Proxy " VERSION " GNU project (C) 1998-99\n");
-	printf ("Coded by James Seter :bugs-> (Pharos@refract.com) or IRC pharos on efnet\n");
+	printf ("Irc Proxy " VERSION " Copyright 1998-2002\n");
+	printf ("Coded by James Seter (pharos@gotbnc.com)\n");
 	printf ("--Using conf file %s\n", conffile);
 
 	memset (&bncconf, 0, sizeof (bncconf));
+	bncconf.logfile = NULL;
 	bncconf.cport = 6667;
 	bncconf.mtype=1;
 	
@@ -173,25 +194,28 @@ int main (int argc, char **argv)
 		bnckill (tmps);
 	}
 
-	printf ("--Configuration:\n");
-	printf ("    Daemon port......:%u\n    Maxusers.........:%u\n    Default conn port:%u\n    Pid File.........:%s\n",
-		bncconf.dport, bncconf.maxusers, bncconf.cport, bncconf.pidfile);
+	printf("--Configuration:\n");
+	if(*bncconf.dhost)
+		printf("    Daemon host......: %s\n", bncconf.dhost);	
+	printf("    Daemon port......: %u\n", bncconf.dport);
+	if(bncconf.maxusers == 0)
+		printf("    Maxusers.........: unlimited\n");
+	else
+		printf("    Maxusers.........: %u\n", bncconf.maxusers);
+	printf("    Default conn port: %u\n", bncconf.cport);
+	printf("    Pid File.........: %s\n", bncconf.pidfile);
 	
 	if (bncconf.vhostdefault[0] == '\0')
-	{
-		printf ("    Vhost Default....:-SYSTEM DEFAULT-\n");
-	}
+		printf("    Vhost Default....: -SYSTEM DEFAULT-\n");
 	else
-	{
-		printf ("    Vhost Default....:%s\n", bncconf.vhostdefault);
-	}
+		printf("    Vhost Default....: %s\n", bncconf.vhostdefault);
 	
 	if (bncconf.vhostlist != NULL)
 	{
 		hockum = bncconf.vhostlist;
 		while (hockum != NULL)
 		{
-			printf ("    Vhost entry......:%s\n", hockum->vhost);
+			printf ("    Vhost entry......: %s\n", hockum->vhost);
 			hockum = hockum->next;
 		}
 	}
@@ -217,7 +241,7 @@ int main (int argc, char **argv)
 		}
 		default:
 		{
-			printf ("    Process Id.......:%i\n", tmps);
+			printf ("    Process Id.......: %i\n", tmps);
 			bnckill (BACKGROUND);
 		}
 	}
@@ -227,7 +251,7 @@ int main (int argc, char **argv)
 		fprintf (mylife, "%i\n", getpid ());
 		fclose (mylife);
 	}
-	logprint(&bncconf, "BNC started. pid %i", getpid ());
+	logprint(&bncconf, "BNC started. pid %i\n", getpid ());
 	tmps = ircproxy (&bncconf);
 	if(tmps)
 	{
